@@ -63,9 +63,7 @@ struct _NMSettingUserClass {
 	NMSettingClass parent;
 };
 
-G_DEFINE_TYPE_WITH_CODE (NMSettingUser, nm_setting_user, NM_TYPE_SETTING,
-                         _nm_register_setting (USER, NM_SETTING_PRIORITY_USER))
-NM_SETTING_REGISTER_TYPE (NM_TYPE_SETTING_USER)
+G_DEFINE_TYPE (NMSettingUser, nm_setting_user, NM_TYPE_SETTING)
 
 #define NM_SETTING_USER_GET_PRIVATE(self) _NM_GET_PRIVATE(self, NMSettingUser, NM_IS_SETTING_USER)
 
@@ -210,7 +208,7 @@ nm_setting_user_check_val (const char *val, GError **error)
 static GHashTable *
 _create_data_hash (void)
 {
-	return g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	return g_hash_table_new_full (nm_str_hash, g_str_equal, g_free, g_free);
 }
 
 /**
@@ -226,7 +224,6 @@ nm_setting_user_get_keys (NMSettingUser *setting, guint *out_len)
 {
 	NMSettingUser *self = setting;
 	NMSettingUserPrivate *priv;
-	guint len;
 
 	g_return_val_if_fail (NM_IS_SETTING_USER (self), NULL);
 
@@ -237,19 +234,13 @@ nm_setting_user_get_keys (NMSettingUser *setting, guint *out_len)
 		return priv->keys;
 	}
 
-	if (!priv->data || !g_hash_table_size (priv->data)) {
-		NM_SET_OUT (out_len, 0);
-		return (const char **) &priv->keys;
-	}
+	priv->keys = nm_utils_strdict_get_keys (priv->data,
+	                                        TRUE,
+	                                        out_len);
 
-	priv->keys = (const char **) g_hash_table_get_keys_as_array (priv->data, &len);
-	g_qsort_with_data (priv->keys,
-	                   len,
-	                   sizeof (const char *),
-	                   nm_strcmp_p_with_data,
-	                   NULL);
-	NM_SET_OUT (out_len, len);
-	return priv->keys;
+	/* don't return %NULL, but hijack the @keys fields as a pseudo
+	 * empty strv array. */
+	return priv->keys ?: ((const char **) &priv->keys);
 }
 
 /*****************************************************************************/
@@ -404,56 +395,34 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 	return TRUE;
 }
 
-static gboolean
-hash_table_equal (GHashTable *a, GHashTable *b)
-{
-	guint n;
-	GHashTableIter iter;
-	const char *key, *value, *valu2;
-
-	n = a ? g_hash_table_size (a) : 0;
-	if (n != (b ? g_hash_table_size (b) : 0))
-		return FALSE;
-	if (n > 0) {
-		g_hash_table_iter_init (&iter, a);
-		while (g_hash_table_iter_next (&iter, (gpointer *) &key, (gpointer *) &value)) {
-			if (!g_hash_table_lookup_extended (b, key, NULL, (gpointer *) &valu2))
-				return FALSE;
-			if (!nm_streq (value, valu2))
-				return FALSE;
-		}
-	}
-	return TRUE;
-
-}
-
-static gboolean
-compare_property (NMSetting *setting,
+static NMTernary
+compare_property (const NMSettInfoSetting *sett_info,
+                  guint property_idx,
+                  NMSetting *setting,
                   NMSetting *other,
-                  const GParamSpec *prop_spec,
                   NMSettingCompareFlags flags)
 {
 	NMSettingUserPrivate *priv, *pri2;
 
-	g_return_val_if_fail (NM_IS_SETTING_USER (setting), FALSE);
-	g_return_val_if_fail (NM_IS_SETTING_USER (other), FALSE);
+	if (nm_streq (sett_info->property_infos[property_idx].name, NM_SETTING_USER_DATA)) {
 
-	if (!nm_streq0 (prop_spec->name, NM_SETTING_USER_DATA))
-		goto call_parent;
+		if (NM_FLAGS_HAS (flags, NM_SETTING_COMPARE_FLAG_INFERRABLE))
+			return NM_TERNARY_DEFAULT;
 
-	priv = NM_SETTING_USER_GET_PRIVATE (NM_SETTING_USER (setting));
-	pri2 = NM_SETTING_USER_GET_PRIVATE (NM_SETTING_USER (other));
+		if (!other)
+			return TRUE;
 
-	if (!hash_table_equal (priv->data, pri2->data))
-		return FALSE;
+		priv = NM_SETTING_USER_GET_PRIVATE (NM_SETTING_USER (setting));
+		pri2 = NM_SETTING_USER_GET_PRIVATE (NM_SETTING_USER (other));
+		return    nm_utils_hash_table_equal (priv->data, pri2->data, TRUE, g_str_equal)
+		       && nm_utils_hash_table_equal (priv->data_invalid, pri2->data_invalid, TRUE, g_str_equal);
+	}
 
-	if (!hash_table_equal (priv->data_invalid, pri2->data_invalid))
-		return FALSE;
-
-	return TRUE;
-
-call_parent:
-	return NM_SETTING_CLASS (nm_setting_user_parent_class)->compare_property (setting, other, prop_spec, flags);
+	return NM_SETTING_CLASS (nm_setting_user_parent_class)->compare_property (sett_info,
+	                                                                          property_idx,
+	                                                                          setting,
+	                                                                          other,
+	                                                                          flags);
 }
 
 /*****************************************************************************/
@@ -579,22 +548,22 @@ nm_setting_user_class_init (NMSettingUserClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	NMSettingClass *setting_class = NM_SETTING_CLASS (klass);
+	GArray *properties_override = _nm_sett_info_property_override_create_array ();
 
-	object_class->set_property = set_property;
 	object_class->get_property = get_property;
+	object_class->set_property = set_property;
 	object_class->finalize     = finalize;
+
 	setting_class->compare_property = compare_property;
-	setting_class->verify = verify;
+	setting_class->verify           = verify;
 
 	/**
-	 * NMSettingUser:data:
+	 * NMSettingUser:data: (type GHashTable(utf8,utf8))
 	 *
 	 * A dictionary of key/value pairs with user data. This data is ignored by NetworkManager
 	 * and can be used at the users discretion. The keys only support a strict ascii format,
 	 * but the values can be arbitrary UTF8 strings up to a certain length.
 	 *
-	 * Type: GHashTable(utf8,utf8)
-	 *
 	 * Since: 1.8
 	 **/
 	/* ---ifcfg-rh---
@@ -615,10 +584,14 @@ nm_setting_user_class_init (NMSettingUserClass *klass)
 	                        G_PARAM_READWRITE |
 	                        G_PARAM_STATIC_STRINGS);
 
+	_properties_override_add_transform (properties_override,
+	                                    obj_properties[PROP_DATA],
+	                                    G_VARIANT_TYPE ("a{ss}"),
+	                                    _nm_utils_strdict_to_dbus,
+	                                    _nm_utils_strdict_from_dbus);
+
 	g_object_class_install_properties (object_class, _PROPERTY_ENUMS_LAST, obj_properties);
 
-	_nm_setting_class_transform_property (setting_class, NM_SETTING_USER_DATA,
-	                                      G_VARIANT_TYPE ("a{ss}"),
-	                                      _nm_utils_strdict_to_dbus,
-	                                      _nm_utils_strdict_from_dbus);
+	_nm_setting_class_commit_full (setting_class, NM_META_SETTING_TYPE_USER,
+	                               NULL, properties_override);
 }
